@@ -68,3 +68,29 @@ checkData = function(self, data, types, B64 = FALSE) {
             variableContainsOneUniqueValue(self, col, colName)
     }
 }
+
+exceptions = list(
+    "attributeError" = "attributeError",
+    "indexError" = "indexError",
+    "keyError" = "keyError",
+    "modelError" = "modelError",
+    "nameError" = "nameError",
+    "valueError" = "valueError",
+    "dataError" = "dataError"
+)
+
+cfaErrors = list(
+    list(
+        originalMessage = paste0(
+            'invalid object for slot "fx.group" in class "Fit": got class "NULL", should be or',
+            ' extend class "numeric"'
+        ),
+        message = "Model dit not converge",
+        class = exceptions$modelError
+    ),
+    list(
+        originalMessage = 'lavaan ERROR: fit measures not available if model did not converge',
+        message = "Model dit not converge",
+        class = exceptions$modelError
+    )
+)
