@@ -3,8 +3,120 @@
 logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
     "logRegOrdClass",
     inherit = logRegOrdBase,
+    #### Active bindings ----
+    active = list(
+        dataProcessed = function() {
+            if (is.null(private$.dataProcessed))
+                private$.dataProcessed <- private$.cleanData()
+
+            return(private$.dataProcessed)
+        },
+        weights = function() {
+            if (is.null(private$.weights))
+                private$.weights <- private$.computeWeights()
+
+            return(private$.weights)
+        },
+        formulas = function() {
+            if (is.null(private$.formulas))
+                private$.formulas <- private$.getFormulas()
+
+            return(private$.formulas)
+        },
+        models = function() {
+            if (is.null(private$.models))
+                private$.models <- private$.computeModels()
+
+            return(private$.models)
+        },
+        nModels = function() {
+            if (is.null(private$.nModels))
+                private$.nModels <- length(self$options$blocks)
+
+            return(private$.nModels)
+        },
+        nullModel = function() {
+            if (is.null(private$.nullModel))
+                private$.nullModel <- private$.computeNullModel()
+
+            return(private$.nullModel)
+        },
+        lrtModelComparison = function() {
+            if (is.null(private$.lrtModelComparison) && self$nModels > 1) {
+                private$.lrtModelComparison <- do.call(
+                    stats::anova,
+                    c(self$models, test="Chisq")
+                )
+            }
+
+            return(private$.lrtModelComparison)
+        },
+        lrtModelTerms = function() {
+            if (is.null(private$.lrtModelTerms))
+                private$.lrtModelTerms <- private$.computeLrtModelTerms()
+
+            return(private$.lrtModelTerms)
+        },
+        deviance = function() {
+            if (is.null(private$.deviance))
+                private$.deviance <- private$.computeDeviance()
+
+            return(private$.deviance)
+        },
+        AIC = function() {
+            if (is.null(private$.AIC))
+                private$.AIC <- private$.computeAIC()
+
+            return(private$.AIC)
+        },
+        BIC = function() {
+            if (is.null(private$.BIC))
+                private$.BIC <- private$.computeBIC()
+
+            return(private$.BIC)
+        },
+        pseudoR2 = function() {
+            if (is.null(private$.pseudoR2))
+                private$.pseudoR2 <- private$.computePseudoR2()
+
+            return(private$.pseudoR2)
+        },
+        modelTest = function() {
+            if (is.null(private$.modelTest))
+                private$.modelTest <- private$.computeModelTest()
+
+            return(private$.modelTest)
+        },
+        CICoefEst = function() {
+            if (is.null(private$.CICoefEst))
+                private$.CICoefEst <- private$.computeCICoefEst()
+
+            return(private$.CICoefEst)
+        },
+        CICoefEstOR = function() {
+            if (is.null(private$.CICoefEstOR))
+                private$.CICoefEstOR <- private$.computeCICoefEst(type="OR")
+
+            return(private$.CICoefEstOR)
+        }
+    ),
     private = list(
         #### Member variables ----
+        .dataProcessed = NULL,
+        .weights = NULL,
+        .models = NULL,
+        .nModels = NULL,
+        .nullModel = NULL,
+        .formulas = NULL,
+        .lrtModelComparison = NULL,
+        .lrtModelTerms = NULL,
+        .deviance = NULL,
+        .AIC = NULL,
+        .BIC = NULL,
+        .pseudoR2 = NULL,
+        .modelTest = NULL,
+        .CICoefEst = NULL,
+        .CICoefEstOR = NULL,
         terms = NULL,
         coefTerms = list(),
         thresTerms = list(),
@@ -12,7 +124,6 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
 
         #### Init + run functions ----
         .init = function() {
-
             private$.modelTerms()
 
             private$.initModelFitTable()
@@ -24,85 +135,143 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
 
         },
         .run = function() {
-
-            ready <- TRUE
-            if (is.null(self$options$dep) || length(self$options$blocks) < 1 || length(self$options$blocks[[1]]) == 0)
-                ready <- FALSE
-
-            if (ready) {
-
-                data <- private$.cleanData()
-                private$.errorCheck(data)
-
-                results <- private$.compute(data)
-
-                private$.populateModelFitTable(results)
-                private$.populateModelCompTable(results)
-                private$.populateLrtTables(results)
-                private$.populateCoefTables(results)
-                private$.populateThresTables(results)
+            if (
+                is.null(self$options$dep) ||
+                length(self$options$blocks) < 1 ||
+                length(self$options$blocks[[1]]) == 0
+            ) {
+                return()
             }
+
+            private$.errorCheck()
+
+            private$.populateModelFitTable()
+            private$.populateModelCompTable()
+            private$.populateLrtTables()
+            private$.populateCoefTables()
+            private$.populateThresTables()
         },
 
         #### Compute results ----
-        .compute = function(data) {
+        .computeModels = function(modelNo = NULL) {
+            data <- self$dataProcessed
+            formulas <- self$formulas
 
-            formulas <- private$.formulas()
+            if (is.numeric(modelNo))
+                formulas <- formulas[modelNo]
 
             globalContr <- options('contrasts')$contrasts
             options('contrasts' = c('contr.treatment', 'contr.poly'))
             on.exit(options('contrasts', substitute(globalContr)), add=TRUE)
 
-            suppressWarnings({
-                suppressMessages({
+            models <- list()
+            for (i in seq_along(formulas)) {
+                models[[i]] <- MASS::polr(
+                    formulas[[i]], data=data, model=TRUE, Hess=TRUE, weights=self$weights
+                )
+                models[[i]]$call$formula <- formulas[[i]]
+            }
 
-                    models <- list(); modelTest <- list(); lrTestTerms <- list(); dev <- list();
-                    AIC <- list(); BIC <- list(); pseudoR <- list(); CI <- list(); CIOR <- list()
-
-                    nullFormula <- as.formula(paste0(jmvcore::toB64(self$options$dep), '~ 1'))
-                    nullModel <- MASS::polr(nullFormula, data=data, model=TRUE, Hess=TRUE)
-                    null <- list(dev=nullModel$deviance, df=nullModel$edf)
-
-                    for (i in seq_along(formulas)) {
-
-                        models[[i]] <- MASS::polr(formulas[[i]], data=data, model=TRUE, Hess=TRUE)
-                        models[[i]]$call$formula <- formulas[[i]]
-
-                        lrTestTerms[[i]] <- car::Anova(models[[i]], test="LR", type=3, singular.ok=TRUE)
-                        modelTest[[i]] <- private$.modelTest(models[[i]], null)
-                        dev[[i]] <- models[[i]]$deviance
-                        AIC[[i]] <- stats::AIC(models[[i]])
-                        BIC[[i]] <- stats::BIC(models[[i]])
-                        pseudoR[[i]] <- private$.pseudoR2(models[[i]], null)
-
-                        CI[[i]] <- try(confint(models[[i]], level=self$options$ciWidth/100), silent=TRUE)
-                        # if (class(CI[[i]]) == 'try-error')
-                        #     CI[[i]] <- confint.default(models[[i]], level=self$options$ciWidth/100)
-
-                        CILO <- try(confint(models[[i]], level=self$options$ciWidthOR/100), silent=TRUE)
-                        # if (class(CILO[[i]]) == 'try-error')
-                        #     CILO <- confint.default(models[[i]], level=self$options$ciWidthOR/100)
-
-                        CIOR[[i]] <- exp(CILO)
-                    }
-
-                    if (length(formulas) > 1)
-                        lrTest <- do.call(stats::anova, c(models, test="Chisq"))
-                    else
-                        lrTest <- NULL
-
-                }) # Supress messages
-            }) # Supress warnings
-
-            results <- list(models=models, modelTest=modelTest, lrTestTerms=lrTestTerms, dev=dev,
-                            AIC=AIC, BIC=BIC, pseudoR=pseudoR, lrTest=lrTest, CI=CI, CIOR=CIOR)
-
-            return(results)
+            return(models)
         },
+        .computeNullModel = function() {
+            nullFormula <- as.formula(paste0(jmvcore::toB64(self$options$dep), '~ 1'))
+            nullModel <-  MASS::polr(
+                nullFormula, data=self$dataProcessed, model=TRUE, Hess=TRUE, weights=self$weights
+            )
+            return(list(dev=nullModel$deviance, df=nullModel$edf))
+        },
+        .computeWeights = function() {
+            global_weights <- attr(self$data, "jmv-weights")
+
+            if (is.null(global_weights))
+                return()
+
+            weights <- self$dataProcessed[[".WEIGHTS"]]
+
+            if (any(weights < 0)) {
+                jmvcore::reject(
+                    .("Weights contains negative values. Negative weights are not permitted.")
+                )
+            }
+
+            return(weights)
+        },
+        .computeLrtModelTerms = function() {
+            lrtModelTerms <- list()
+            for (i in seq_len(self$nModels)) {
+                lrtModelTerms[[i]] <- car::Anova(
+                    self$models[[i]],
+                    test="LR",
+                    type=3,
+                    singular.ok=TRUE
+                )
+            }
+            return(lrtModelTerms)
+        },
+        .computeDeviance = function() {
+            dev <- list()
+            for (i in seq_len(self$nModels))
+                dev[[i]] <- self$models[[i]]$deviance
+
+            return(dev)
+        },
+        .computeAIC = function() {
+            AIC <- list()
+            for (i in seq_len(self$nModels))
+                AIC[[i]] <- stats::AIC(self$models[[i]])
+
+            return(AIC)
+        },
+        .computeBIC = function() {
+            BIC <- list()
+            for (i in seq_len(self$nModels))
+                BIC[[i]] <- stats::BIC(self$models[[i]])
+
+            return(BIC)
+        },
+        .computePseudoR2 = function() {
+            pR2 <- list()
+            for (i in seq_len(self$nModels)) {
+                dev <- self$deviance[[i]]
+                n <- length(self$models[[i]]$fitted.values)
+
+                r2mf <- 1 - dev / self$nullModel$dev
+                r2cs <- 1 - exp(-(self$nullModel$dev - dev) / n)
+                r2n <- r2cs / (1 - exp(-self$nullModel$dev / n))
+
+                pR2[[i]] <- list(r2mf=r2mf, r2cs=r2cs, r2n=r2n)
+            }
+            return(pR2)
+        },
+        .computeModelTest = function() {
+            modelTest <- list()
+            for (i in seq_len(self$nModels)) {
+                chi <- self$nullModel$dev - self$models[[i]]$deviance
+                df <- abs(self$nullModel$df - self$models[[i]]$edf)
+                p <- 1 - pchisq(chi, df)
+                modelTest[[i]] <- list(chi=chi, df=df, p=p)
+            }
+            return(modelTest)
+        },
+        .computeCICoefEst = function(type="LOR") {
+            if (type == "OR")
+                level <- self$options$ciWidthOR / 100
+            else
+                level <- self$options$ciWidth / 100
+
+            ci <- list()
+            for (i in seq_len(self$nModels)) {
+                ci[[i]] <- confint(self$models[[i]], level=level)
+                if (type == "OR")
+                    ci[[i]] <- exp(ci[[i]])
+            }
+            return(ci)
+        },
+
 
         #### Init tables/plots functions ----
         .initModelFitTable = function() {
-
             table <- self$results$modelFit
 
             for (i in seq_along(self$options$blocks))
@@ -126,7 +295,6 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             )
         },
         .initModelCompTable = function() {
-
             table <- self$results$modelComp
             terms <- private$terms
 
@@ -137,10 +305,8 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
 
             for (i in 1:(length(terms)-1))
                 table$addRow(rowKey=i, values=list(model1 = i, model2 = as.integer(i+1)))
-
         },
         .initModelSpec = function() {
-
             groups <- self$results$models
 
             for (i in seq_along(self$options$blocks)) {
@@ -150,7 +316,6 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             }
         },
         .initLrtTables = function() {
-
             groups <- self$results$models
             termsAll <- private$terms
 
@@ -159,12 +324,15 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
                 table <- groups$get(key=i)$lrt
                 terms <- termsAll[[i]]
 
-                for (j in seq_along(terms))
-                    table$addRow(rowKey=paste0(terms[[j]]), values=list(term = jmvcore::stringifyTerm(terms[j])))
+                for (j in seq_along(terms)) {
+                    table$addRow(
+                        rowKey=paste0(terms[[j]]),
+                        values=list(term = jmvcore::stringifyTerm(terms[j]))
+                    )
+                }
             }
         },
         .initCoefTables = function() {
-
             groups <- self$results$models
             termsAll <- private$terms
             data <- self$data
@@ -184,7 +352,6 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             ciWidthORTitle <- jmvcore::format(ciWidthTitleString, ciWidth=self$options$ciWidthOR)
 
             for (i in seq_along(termsAll)) {
-
                 table <- groups$get(key=i)$coef
 
                 table$getColumn('lower')$setSuperTitle(ciWidthTitle)
@@ -197,12 +364,15 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
                 terms <- termsAll[[i]]
 
                 for (j in seq_along(terms)) {
-
                     if (any(terms[[j]] %in% factors)) { # check if there are factors in the term
-
-                        table$addRow(rowKey=terms[[j]], values=list(term = paste0(jmvcore::stringifyTerm(terms[[j]]), ':'),
-                                                                    est='', se='', odds='', z='', p='',
-                                                                    lower='', upper='', oddsLower='', oddsUpper=''))
+                        table$addRow(
+                            rowKey=terms[[j]],
+                            values=list(
+                                term = paste0(jmvcore::stringifyTerm(terms[[j]]), ':'),
+                                est='', se='', odds='', z='', p='', lower='', upper='',
+                                oddsLower='', oddsUpper=''
+                            )
+                        )
 
                         coefs <- private$.coefTerms(terms[[j]])
                         coefNames <- coefs$coefNames
@@ -214,14 +384,13 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
                         }
 
                         coefTerms <- c(coefTerms, coefs$coefTerms)
-
                     } else {
-
                         rowKey <- jmvcore::composeTerm(jmvcore::toB64(terms[[j]]))
-                        table$addRow(rowKey=rowKey, values=list(term = jmvcore::stringifyTerm(terms[[j]])))
+                        table$addRow(
+                            rowKey=rowKey, values=list(term = jmvcore::stringifyTerm(terms[[j]]))
+                        )
 
                         coefTerms[[length(coefTerms) + 1]] <- jmvcore::toB64(terms[[j]])
-
                     }
                 }
 
@@ -229,7 +398,6 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             }
         },
         .initThresTables = function() {
-
             groups <- self$results$models
             termsAll <- private$terms
             data <- self$data
@@ -243,13 +411,10 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             }
 
             for (i in seq_along(termsAll)) {
-
                 table <- groups$get(key=i)$thres
 
                 thresTerms <- list()
-
                 for ( k in 1:(length(depLevels) - 1) ) {
-
                     rowKey <- paste0(jmvcore::toB64(depLevels[k]), '|', jmvcore::toB64(depLevels[k + 1]))
                     rowName <- paste0(depLevels[k], ' | ', depLevels[k + 1])
                     table$addRow(rowKey=rowKey, values=list(term = rowName))
@@ -261,75 +426,60 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
         },
 
         #### Populate tables functions ----
-        .populateModelFitTable = function(results) {
-
+        .populateModelFitTable = function() {
             table <- self$results$modelFit
-
-            AIC <- results$AIC
-            BIC <- results$BIC
-            pR2 <- results$pseudoR
-            modelTest <- results$modelTest
-            dev <- results$dev
-
-            for (i in seq_along(AIC)) {
-
+            for (i in seq_len(self$nModels)) {
                 row <- list()
-                row[["r2mf"]] <- pR2[[i]]$r2mf
-                row[["r2cs"]] <- pR2[[i]]$r2cs
-                row[["r2n"]] <- pR2[[i]]$r2n
-                row[["dev"]] <- dev[[i]]
-                row[["aic"]] <- AIC[[i]]
-                row[["bic"]] <- BIC[[i]]
-                row[["chi"]] <- modelTest[[i]]$chi
-                row[["df"]] <- modelTest[[i]]$df
-                row[["p"]] <- modelTest[[i]]$p
+
+                row[["r2mf"]] <- self$pseudoR2[[i]]$r2mf
+                row[["r2cs"]] <- self$pseudoR2[[i]]$r2cs
+                row[["r2n"]] <- self$pseudoR2[[i]]$r2n
+                row[["dev"]] <- self$deviance[[i]]
+                row[["aic"]] <- self$AIC[[i]]
+                row[["bic"]] <- self$BIC[[i]]
+                row[["chi"]] <- self$modelTest[[i]]$chi
+                row[["df"]] <- self$modelTest[[i]]$df
+                row[["p"]] <- self$modelTest[[i]]$p
 
                 table$setRow(rowNo=i, values = row)
             }
         },
-        .populateModelCompTable = function(results) {
-
-            table <- self$results$modelComp
-
-            models <- results$models
-            lrTest <- results$lrTest
-            r <- lrTest[-1,]
-
-            if (length(models) <= 1)
+        .populateModelCompTable = function() {
+            if (length(self$nModels) <= 1)
                 return()
 
-            for (i in 1:(length(models)-1)) {
+            table <- self$results$modelComp
+            r <- self$lrtModelComparison[-1,]
 
+            for (i in seq_len(self$nModel - 1)) {
                 row <- list()
                 row[["chi"]] <- r[['LR stat.']][i]
                 row[["df"]] <- r[['   Df']][i]
                 row[["p"]] <- r[['Pr(Chi)']][i]
 
-                table$setRow(rowNo=i, values = row)
+                table$setRow(rowNo=i, values=row)
             }
         },
-        .populateLrtTables = function(results) {
-
+        .populateLrtTables = function() {
             groups <- self$results$models
             termsAll <- private$terms
-            lrTests <- results$lrTestTerms
 
             for (i in seq_along(termsAll)) {
-
                 table <- groups$get(key=i)$lrt
 
                 terms <- termsAll[[i]]
                 termsB64 <- lapply(terms, jmvcore::toB64)
-                lrt <- lrTests[[i]]
+                lrt <- self$lrtModelTerms[[i]]
                 rowTerms <- jmvcore::decomposeTerms(rownames(lrt))
 
                 for (j in seq_along(terms)) {
-
                     term <- termsB64[[j]]
 
                     # check which rows have the same length + same terms
-                    index <- which(length(term) == sapply(rowTerms, length) &
-                                       sapply(rowTerms, function(x) all(term %in% x)))
+                    index <- which(
+                        length(term) == sapply(rowTerms, length) &
+                            sapply(rowTerms, function(x) all(term %in% x))
+                    )
 
                     row <- list()
                     row[["chi"]] <- lrt[index, 'LR Chisq']
@@ -340,21 +490,18 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
                 }
             }
         },
-        .populateCoefTables = function(results) {
-
+        .populateCoefTables = function() {
             groups <- self$results$models
             termsAll <- private$coefTerms
-            models <- results$models
 
             for (i in seq_along(termsAll)) {
-
                 table <- groups$get(key=i)$coef
 
-                model <- summary(models[[i]])
-                CI <- results$CI[[i]]
-                CIOR <- results$CIOR[[i]]
-                coef<- model$coefficients[,1]
-                se <- model$coefficients[,2]
+                model <- summary(self$models[[i]])
+                CI <- self$CICoefEst[[i]]
+                CIOR <- self$CICoefEstOR[[i]]
+                coef<- model$coefficients[, 1]
+                se <- model$coefficients[, 2]
                 wald <- model$coefficients[,3]
                 p <- (1 - pnorm(abs(wald), 0, 1)) * 2
 
@@ -362,11 +509,12 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
                 rowTerms <- jmvcore::decomposeTerms(names(coef))
 
                 for (k in seq_along(terms)) {
-
                     term <- terms[[k]]
 
-                    index <- which(length(term) == sapply(rowTerms, length) &
-                                       sapply(rowTerms, function(x) all(term %in% x)))
+                    index <- which(
+                        length(term) == sapply(rowTerms, length) &
+                            sapply(rowTerms, function(x) all(term %in% x))
+                    )
 
                     row <- list()
                     row[["est"]] <- coef[index]
@@ -387,22 +535,18 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
                         row[["oddsUpper"]] <- CIOR[index, 2]
                     }
 
-
                     table$setRow(rowKey=jmvcore::composeTerm(terms[[k]]), values = row)
                 }
             }
         },
-        .populateThresTables = function(results) {
-
+        .populateThresTables = function() {
             groups <- self$results$models
             termsAll <- private$thresTerms
-            models <- results$models
 
             for (i in seq_along(termsAll)) {
-
                 table <- groups$get(key=i)$thres
 
-                model <- summary(models[[i]])
+                model <- summary(self$models[[i]])
                 coef<- model$coefficients[,1]
                 se <- model$coefficients[,2]
                 wald <- model$coefficients[,3]
@@ -412,7 +556,6 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
                 rowTerms <- names(coef)
 
                 for (k in seq_along(terms)) {
-
                     term <- terms[[k]]
 
                     index <- which(term == rowTerms)
@@ -431,7 +574,6 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
 
         #### Helper functions ----
         .modelTerms = function() {
-
             blocks <- self$options$blocks
 
             terms <- list()
@@ -445,10 +587,8 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             }
 
             private$terms <- terms
-
         },
         .coefTerms = function(terms) {
-
             covs <- self$options$covs
             factors <- self$options$factors
             refLevels <- self$options$refLevels
@@ -470,18 +610,24 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
                     contrLevels[[term]] <- levels[[term]][-refNo]
                     refLevel[[term]] <- levels[[term]][refNo]
 
-                    if (length(terms) > 1)
-                        contr[[term]] <- paste0('(', paste(contrLevels[[term]], refLevel[[term]], sep = ' \u2013 '), ')')
-                    else
-                        contr[[term]] <- paste(contrLevels[[term]], refLevel[[term]], sep = ' \u2013 ')
-
-                    rContr[[term]] <- paste0(jmvcore::toB64(term), jmvcore::toB64(contrLevels[[term]]))
+                    if (length(terms) > 1) {
+                        contr[[term]] <- paste0(
+                            '(',
+                            paste(contrLevels[[term]], refLevel[[term]], sep = ' \u2013 '),
+                            ')'
+                        )
+                    } else {
+                        contr[[term]] <- paste(
+                            contrLevels[[term]], refLevel[[term]], sep = ' \u2013 '
+                        )
+                    }
 
+                    rContr[[term]] <- paste0(
+                        jmvcore::toB64(term), jmvcore::toB64(contrLevels[[term]])
+                    )
                 } else {
-
                     contr[[term]] <- term
                     rContr[[term]] <- jmvcore::toB64(term)
-
                 }
             }
 
@@ -495,8 +641,7 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
 
             return(list(coefNames=coefNames, coefTerms=coefTerms))
         },
-        .formulas = function() {
-
+        .getFormulas = function() {
             dep <- self$options$dep
             depB64 <- jmvcore::toB64(dep)
             terms <- private$terms
@@ -505,15 +650,16 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             for (i in seq_along(terms)) {
                 termsB64 <- lapply(terms[[i]], jmvcore::toB64)
                 composedTerms <- jmvcore::composeTerms(termsB64)
-                formulas[[i]] <- as.formula(paste(depB64, paste0(composedTerms, collapse ="+"), sep="~"))
+                formulas[[i]] <- as.formula(
+                    paste(depB64, paste0(composedTerms, collapse ="+"), sep="~")
+                )
             }
 
             return(formulas)
         },
-        .errorCheck = function(data) {
-
+        .errorCheck = function() {
             dep <- self$options$dep
-            column <- data[[jmvcore::toB64(dep)]]
+            column <- self$dataProcessed[[jmvcore::toB64(dep)]]
 
             if (length(levels(column)) == 2) {
                 jmvcore::reject(
@@ -526,7 +672,6 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             }
         },
         .cleanData = function() {
-
             dep <- self$options$dep
             covs <- self$options$covs
             factors <- self$options$factors
@@ -536,13 +681,14 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
 
             data <- list()
 
-            data[[jmvcore::toB64(dep)]] <- factor(jmvcore::toB64(as.character(dataRaw[[dep]])),
-                                                  levels=jmvcore::toB64(levels(dataRaw[[dep]])))
+            data[[jmvcore::toB64(dep)]] <- factor(
+                jmvcore::toB64(as.character(dataRaw[[dep]])),
+                levels=jmvcore::toB64(levels(dataRaw[[dep]]))
+            )
 
             refVars <- sapply(refLevels, function(x) x$var)
 
             for (factor in factors) {
-
                 ref <- refLevels[[which(factor == refVars)]][['ref']]
 
                 rows <- jmvcore::toB64(as.character(dataRaw[[factor]]))
@@ -558,6 +704,10 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             for (cov in covs)
                 data[[jmvcore::toB64(cov)]] <- jmvcore::toNumeric(dataRaw[[cov]])
 
+            global_weights <- attr(dataRaw, "jmv-weights")
+            if (! is.null(global_weights))
+                data[[".WEIGHTS"]] <- jmvcore::toNumeric(global_weights)
+
             attr(data, 'row.names') <- seq_len(length(data[[1]]))
             attr(data, 'class') <- 'data.frame'
             data <- jmvcore::naOmit(data)
@@ -565,7 +715,6 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             return(data)
         },
         .createContrasts=function(levels) {
-
             nLevels <- length(levels)
 
             dummy <- contr.treatment(levels)
@@ -574,24 +723,5 @@ logRegOrdClass <- if (requireNamespace('jmvcore')) R6::R6Class(
             contrast <- (dummy - coding)
 
             return(contrast)
-        },
-        .pseudoR2 = function(model, null) {
-
-            dev <- model$deviance
-            n <- length(model$fitted.values)
-
-            r2mf <- 1 - dev/null$dev
-            r2cs <- 1 - exp(-(null$dev - dev) / n)
-            r2n <- r2cs / (1 - exp(-null$dev / n))
-
-            return(list(r2mf=r2mf, r2cs=r2cs, r2n=r2n))
-        },
-        .modelTest = function(model, null) {
-
-            chi <- null$dev - model$deviance
-            df <- abs(null$df - model$edf)
-            p <- 1 - pchisq(chi, df)
-
-            return(list(chi=chi, df=df, p=p))
         })
 )
