@@ -126,7 +126,7 @@ ttestISClass <- R6::R6Class(
                     else if (any(is.infinite(dataTTest$dep)))
                         res <- createError(.('Variable contains infinite values'))
                     else
-                        res <- try(t.test(dep ~ group, data=dataTTest, var.equal=TRUE, paired=FALSE,
+                        res <- try(t.test(dep ~ group, data=dataTTest, var.equal=TRUE,
                                           alternative=Ha, conf.level=confInt), silent=TRUE)
 
                     if (isError(res)) {
@@ -181,7 +181,7 @@ ttestISClass <- R6::R6Class(
                     else if (any(is.infinite(dataTTest$dep)))
                         res <- createError(.('Variable contains infinite values'))
                     else
-                        res <- try(t.test(dep ~ group, data=dataTTest, var.equal=FALSE, paired=FALSE,
+                        res <- try(t.test(dep ~ group, data=dataTTest, var.equal=FALSE,
                                           alternative=Ha, conf.level=confInt), silent=TRUE)
 
                     if ( ! isError(res)) {
@@ -281,8 +281,7 @@ ttestISClass <- R6::R6Class(
                         if ( ! is.na(m1) && m2 < m1)
                             res <- res2
 
-                        biSerial <- 1 - (2 * res$statistic / (n[1] * n[2]))
-
+                        biSerial <- 1 - (2 * m1 / (n[1] * n[2]))
                     }
 
                     if ( ! isError(res)) {
